Attribute VB_Name = "Registry"
'---------------------------------------------------------------------
' Copyright  2000-2001 SPACE.com Inc., New York, NY
'
' Permission is hereby granted to use this Software for any purpose
' including combining with commercial products, creating derivative
' works, and redistribution of source or binary code, without
' limitation or consideration. Any redistributed copies of this
' Software must include the above Copyright Notice.
'
' THIS SOFTWARE IS PROVIDED "AS IS". SPACE.COM, INC. MAKES NO
' WARRANTIES REGARDING THIS SOFTWARE, EXPRESS OR IMPLIED, AS TO ITS
' SUITABILITY OR FITNESS FOR A PARTICULAR PURPOSE.
'---------------------------------------------------------------------
'   ============
'   REGISTRY.BAS
'   ============
'
' Note you must have a global ALERT_TITLE string for use in
' MsgBox calls.
'
' Written:  28-Jul-00   Robert B. Denny <rdenny@dc3.com>
'
' Edits:
'
' When      Who     What
' --------- ---     --------------------------------------------------
' 28-Jul-00 rbd     Initial edit, from the code in ACP.
' -----------------------------------------------------------------------------
Option Explicit

' =========================
' ROOT OF OUR REGISTRY AREA
' =========================
'
Private Const ROOT_KEY As String = "Software\ASCOM\Telescope Drivers\DummyScopeFor909.Telescope"

' Reg Key Security Options...
Private Const READ_CONTROL As Long = &H20000
Private Const KEY_QUERY_VALUE As Long = &H1
Private Const KEY_SET_VALUE As Long = &H2
Private Const KEY_CREATE_SUB_KEY As Long = &H4
Private Const KEY_ENUMERATE_SUB_KEYS As Long = &H8
Private Const KEY_NOTIFY As Long = &H10
Private Const KEY_CREATE_LINK As Long = &H20
'
Public Const KEY_READ As Long = KEY_QUERY_VALUE + KEY_ENUMERATE_SUB_KEYS + _
                        READ_CONTROL
Public Const KEY_WRITE As Long = KEY_SET_VALUE + KEY_CREATE_SUB_KEY + _
                        READ_CONTROL
Public Const KEY_ALL_ACCESS As Long = KEY_QUERY_VALUE + KEY_SET_VALUE + _
                       KEY_CREATE_SUB_KEY + KEY_ENUMERATE_SUB_KEYS + _
                       KEY_NOTIFY + KEY_CREATE_LINK + READ_CONTROL

Public Const HKEY_CLASSES_ROOT As Long = &H80000000
Public Const HKEY_LOCAL_MACHINE As Long = &H80000002
Public Const HKEY_CURRENT_USER As Long = &H80000001

Public Const REG_SZ As Long = 1                    ' Unicode nul terminated string
Public Const REG_EXPAND_SZ As Long = 2             ' REG_SZ with env var references
Public Const REG_DWORD As Long = 4                 ' 32-bit number

Public Const REG_OPTION_NON_VOLATILE As Long = 0
Public Const REG_CREATED_NEW_KEY As Long = 1


Public Const ENUM_BUF_SIZE = 32768             ' Key enumeration buffer, see GetProfile()

'
' lpClass may be Null so we define it as Any
'

Declare Function RegCreateKeyEx Lib "advapi32" Alias "RegCreateKeyExA" _
   (ByVal hKey As Long, _
    ByVal lpSubKey As String, _
    ByVal dwReserved As Long, _
    ByVal lpClass As String, _
    ByVal dwOptions As Long, _
    ByVal samDesired As Long, _
    ByVal lpSecurityAttributes As Any, _
    ByRef phkResult As Long, _
    ByRef lpDisposition As Long) As Long
    
Declare Function RegOpenKeyEx Lib "advapi32" Alias "RegOpenKeyExA" _
   (ByVal hKey As Long, _
    ByVal lpSubKey As String, _
    ByVal dwReserved As Long, _
    ByVal samDesired As Long, _
    ByRef phkResult As Long) As Long

Declare Function RegEnumValue Lib "advapi32" Alias "RegEnumValueA" _
   (ByVal hKey As Long, _
    ByVal dwIndex As Long, _
    ByVal lpdwName As String, _
    ByRef dwNameLen As Long, _
    ByVal lpReserved As Long, _
    ByRef lpdwType As Long, _
    ByVal lpszValue As String, _
    ByRef lpdwValLen As Long) As Long

Declare Function RegQueryValueEx Lib "advapi32" Alias "RegQueryValueExA" _
   (ByVal hKey As Long, _
    ByVal lpValueName$, _
    ByVal lpdwReserved As Long, _
    ByRef lpdwType As Long, _
    ByVal lpData As String, _
    ByRef lpcbData As Long) As Long
    
Declare Function RegSetValueEx Lib "advapi32" Alias "RegSetValueExA" _
   (ByVal hKey As Long, _
    ByVal lpValueName$, _
    ByVal lpdwReserved As Long, _
    ByVal dwType As Long, _
    ByVal lpData As String, _
    ByVal cbData As Long) As Long

Declare Function RegDeleteValue Lib "advapi32" Alias "RegDeleteValueA" _
    (ByVal hKey As Long, ByVal lpszName As String) As Long
    
Declare Function RegCloseKey Lib "advapi32" _
   (ByVal hKey As Long) As Long
   
Declare Function ExpandEnvironmentStrings Lib "kernel32" _
                Alias "ExpandEnvironmentStringsA" _
                (ByVal lpSrc As String, _
                 ByVal lpDst As String, _
                 ByVal dwSize As Long) As Long

Public hKeyRoot As Long                ' Handle to our registry area


'---------------------------------------------------------------------------
'
'   GetProfile() - Get a value from the root part of the registry
'
'---------------------------------------------------------------------------
Public Function GetProfile(sSection As String, sKey As String) As String

    GetProfile = GetRegString(hKeyRoot, sSection, sKey)
    
End Function

'---------------------------------------------------------------------------
'
'   WriteProfile() - Write/Create a named value in the root part registry
'
'---------------------------------------------------------------------------
Public Sub WriteProfile(sect As String, key As String, val As String)

    WriteRegString hKeyRoot, sect, key, val
        
End Sub

'---------------------------------------------------------------------------
'
'   GetRegString() - Get a string value from the registry
'
' Expand REG_EXPAND_SZ automatically
'---------------------------------------------------------------------------
Public Function GetRegString(hKeyBase As Long, sSection As String, sKey As String) As String
    Dim hKey As Long
    Dim dwType As Long
    Dim dwLen As Long
    Dim szBuf As String

    InitReg                                     ' Opem master key if needed
    GetRegString = ""                           ' Assume failure
    dwType = -1
    
    If sSection = "" Then
        hKey = hKeyBase
    Else
        If RegOpenKeyEx(hKeyBase, sSection, 0, KEY_READ, hKey) <> 0 Then
            Exit Function
        End If
    End If
    
    dwLen = 1023
    szBuf = String$(dwLen, Chr$(0))
    If RegQueryValueEx(hKey, sKey, 0, dwType, szBuf, dwLen) = 0 Then
        If (Asc(Mid(szBuf, dwLen, 1)) = 0) Then     ' Win95 Adds Null Terminated String...
            GetRegString = Left(szBuf, dwLen - 1)   ' Null Found, Extract From String
        Else                                        ' WinNT Does NOT Null Terminate String...
            GetRegString = Left(szBuf, dwLen)       ' Null Not Found, Extract String Only
        End If
    End If
    
    If hKey <> hKeyBase Then RegCloseKey (hKey)
    '
    ' Expand environment strings if needed
    '
    If dwType = REG_EXPAND_SZ Then                  ' If needs env var expansion
        szBuf = String$(1024, Chr$(0))
        dwLen = ExpandEnvironmentStrings(GetRegString, szBuf, 1023)
        GetRegString = Left$(szBuf, (dwLen - 1))    ' Len includes null
    End If
    
End Function

'---------------------------------------------------------------------------
'
'   GetRegDword() - Get a DWORD (long) value from the registry
'
'---------------------------------------------------------------------------
Public Function GetRegDword(hKeyBase As Long, sSection As String, sName As String) As Long
    Dim hKey As Long
    Dim dwType As Long
    Dim dwLen As Long
    Dim szBuf As String
    Dim i As Integer
    Dim keyVal As String

    InitReg                                 ' Opem master key if needed
    GetRegDword = 0
        
    If sSection = "" Then
        hKey = hKeyBase
    Else
        If RegOpenKeyEx(hKeyBase, sSection, 0, KEY_READ, hKey) <> 0 Then
            Exit Function
        End If
    End If
    
    dwLen = 4
    szBuf = String$(dwLen, 0)
    If RegQueryValueEx(hKey, sName, 0, dwType, szBuf, dwLen) = 0 Then
        If (Asc(Mid(szBuf, dwLen, 1)) = 0) Then     ' Win95 Adds Null Terminated String...
            szBuf = Left(szBuf, dwLen - 1)          ' Null Found, Extract From String
        Else                                        ' WinNT Does NOT Null Terminate String...
            szBuf = Left(szBuf, dwLen)              ' Null Not Found, Extract String Only
        End If
    End If
    If hKey <> hKeyBase Then RegCloseKey (hKey)
   
    For i = Len(szBuf) To 1 Step -1                  ' Convert Each Byte
        keyVal = keyVal + Hex(Asc(Mid(szBuf, i, 1))) ' Build Value Char. By Char.
    Next
    GetRegDword = CLng("&H" + keyVal)
    
End Function
'---------------------------------------------------------------------------
'
'   WriteRegString() - Write/Create a string in the registry
'
'---------------------------------------------------------------------------
Public Sub WriteRegString(hKeyBase As Long, sect As String, key As String, val As String)
    Dim hKey As Long
    Dim l As Long
    Dim lRes As Long
    
    '
    ' WHY????? Unless I do this, if val comes from the 'text' property
    ' of a textbox control, the RegSetValueEx returns Win32 error 87
    ' (Invalid parameter). Maybe it's variant vs real string.
    '
    If val = "" Then val = ""
    
    InitReg                                 ' Opem master key if needed
    
    If sect = "" Then
        hKey = hKeyBase
    Else
        If RegCreateKeyEx(hKeyBase, sect, 0&, 0&, 0&, KEY_WRITE, 0&, hKey, lRes) <> 0 Then
            MsgBox "Failed to open/create section key " & sect & ".", _
                    (vbOKOnly + vbExclamation + vbMsgBoxSetForeground), ALERT_TITLE
            End
        End If
    End If
    
    l = (Len(val) + 1)
    If RegSetValueEx(hKey, key, 0, REG_SZ, val, l) <> 0 Then
        MsgBox "Failed to write config value " & key & " to registry section " & _
                sect & ".", (vbOK + vbCritical + vbMsgBoxSetForeground), ALERT_TITLE
        End
    End If
    
    If hKey <> hKeyBase Then RegCloseKey (hKey)
    
End Sub
'
' Open the master registry key if needed
'
Private Sub InitReg()
    Dim lRes As Long, lStat As Long
    Dim c As String
    
    If hKeyRoot <> 0 Then Exit Sub
    
    lStat = RegCreateKeyEx(HKEY_LOCAL_MACHINE, ROOT_KEY, 0&, 0&, 0&, _
                    (KEY_READ + KEY_WRITE + KEY_CREATE_SUB_KEY), 0&, hKeyRoot, lRes)
    If lStat <> 0 Then
        MsgBox "Failed to open/create root registry key (" & lStat & ")", _
                (vbOK + vbCritical + vbMsgBoxSetForeground), ALERT_TITLE
        End
    End If
End Sub


